@@ -7,19 +7,22 @@ from typing import List, Optional, Type, Union
 
 from django.contrib.gis.db import models as gis_models
 from django.contrib.gis.db.models import Extent, GeometryField
-from django.contrib.gis.db.models.functions import Transform
 from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
 from django.db import models
 from django.db.models.fields.reverse_related import ForeignObjectRel
 from django.utils.functional import cached_property  # py3.8: functools
 
+from gisserver.db import conditional_transform
 from gisserver.exceptions import ExternalValueError
 from gisserver.types import (
+    GmlBoundedByElement,
+    GmlNameElement,
     XPathMatch,
     XsdAnyType,
     XsdComplexType,
     XsdElement,
     XsdTypes,
+    GmlIdAttribute,
 )
 from gisserver.geometries import BoundingBox, CRS, WGS84
 
@@ -145,7 +148,7 @@ class FeatureField:
     an XSD definition that the remaining application uses.
     """
 
-    #: The class that's used to generate the XsdElement.
+    #: Allow to override the XSD element type that this field will generate.
     xsd_element_class: Type[XsdElement] = XsdElement
 
     model: Optional[Type[models.Model]]
@@ -157,12 +160,15 @@ class FeatureField:
         model_attribute=None,
         model=None,
         parent: "Optional[ComplexFeatureField]" = None,
+        abstract=None,
     ):
         self.name = name
         self.model_attribute = model_attribute
         self.model = None
         self.model_field = None
         self.parent = parent
+        self.abstract = abstract
+
         self._nillable_relation = False
         if model is not None:
             self.bind(model)
@@ -174,7 +180,9 @@ class FeatureField:
         return get_basic_field_type(self.name, self.model_field)
 
     def bind(
-        self, model: Type[models.Model], parent: "Optional[ComplexFeatureField]" = None,
+        self,
+        model: Type[models.Model],
+        parent: "Optional[ComplexFeatureField]" = None,
     ):
         """Late-binding for the model.
 
@@ -244,7 +252,12 @@ class ComplexFeatureField(FeatureField):
     """
 
     def __init__(
-        self, name: str, fields: _FieldDefinitions, model_attribute=None, model=None
+        self,
+        name: str,
+        fields: _FieldDefinitions,
+        model_attribute=None,
+        model=None,
+        abstract=None,
     ):
         """
         :param name: Name of the model field.
@@ -252,15 +265,31 @@ class ComplexFeatureField(FeatureField):
             be a list of :class:`FeatureField` objects, or plain field names.
             Using ``__all__`` also works but is not recommended outside testing.
         """
-        super().__init__(name, model_attribute=model_attribute, model=model)
+        super().__init__(
+            name, model_attribute=model_attribute, model=model, abstract=abstract
+        )
         self._fields = fields
 
+    @cached_property
+    def fields(self) -> List[FeatureField]:
+        """Provide all fields that will be rendered as part of this complex field."""
+        return _get_model_fields(self.target_model, self._fields, parent=self)
+
     def _get_xsd_type(self) -> XsdComplexType:
         """Generate the XSD description for the field with an object relation."""
-        fields = _get_model_fields(self.target_model, self._fields, parent=self)
+        pk_field = self.target_model._meta.pk
+
         return XsdComplexType(
             name=f"{self.target_model._meta.object_name}Type",
-            elements=[field.xsd_element for field in fields],
+            elements=[field.xsd_element for field in self.fields],
+            attributes=[
+                # Add gml:id attribute definition so it can be resolved in xpath
+                GmlIdAttribute(
+                    type_name=self.name,
+                    source=pk_field,
+                    model_attribute=pk_field.name,
+                )
+            ],
             source=self.target_model,
         )
 
@@ -280,7 +309,11 @@ class ComplexFeatureField(FeatureField):
 
 
 def field(
-    name: str, *, model_attribute=None, fields: Optional[_FieldDefinitions] = None
+    name: str,
+    *,
+    model_attribute=None,
+    abstract: Optional[str] = None,
+    fields: Optional[_FieldDefinitions] = None,
 ) -> FeatureField:
     """Shortcut to define a WFS field.
 
@@ -294,10 +327,10 @@ def field(
     """
     if fields is not None:
         return ComplexFeatureField(
-            name=name, model_attribute=model_attribute, fields=fields
+            name=name, model_attribute=model_attribute, fields=fields, abstract=abstract
         )
     else:
-        return FeatureField(name, model_attribute=model_attribute)
+        return FeatureField(name, model_attribute=model_attribute, abstract=abstract)
 
 
 class FeatureType:
@@ -310,11 +343,15 @@ class FeatureType:
     This corresponds with a single Django model.
     """
 
+    #: Allow to override the XSD complex type that this feature will generate.
+    xsd_type_class: Type[XsdComplexType] = XsdComplexType
+
     def __init__(
         self,
         queryset: models.QuerySet,
         *,
         fields: Optional[_FieldDefinitions] = None,
+        display_field_name: str = None,
         geometry_field_name: str = None,
         name: str = None,
         # WFS Metadata:
@@ -324,12 +361,16 @@ class FeatureType:
         crs: CRS = None,
         other_crs: List[CRS] = None,
         metadata_url: Optional[str] = None,
+        # Settings
+        show_name_field: bool = True,
+        xml_prefix: str = "app",
     ):
         """
         :param queryset: The queryset to retrieve the data.
         :param fields: Define which fields to show in the WFS data.
             This can be a list of field names, or :class:`FeatureField` objects.
-        :param geometry_field_name: Name of the geometry field to expose (default = auto detect).
+        :param display_field_name: Name of the field that's used as general string representation.
+        :param geometry_field: Name of the geometry field to expose (default = auto detect).
         :param name: Name, also used as XML tag name.
         :param title: Used in WFS metadata.
         :param abstract: Used in WFS metadata.
@@ -337,6 +378,9 @@ class FeatureType:
         :param crs: Used in WFS metadata.
         :param other_crs: Used in WFS metadata.
         :param metadata_url: Used in WFS metadata.
+        :param show_name_field: Whether to show the ``gml:name`` or the GeoJSON ``geometry_name``
+            field. Default is to show a field when ``name_field`` is given.
+        :param xml_prefix: The XML namespace prefix to use.
         """
         if isinstance(queryset, models.QuerySet):
             self.queryset = queryset
@@ -349,6 +393,7 @@ class FeatureType:
             raise TypeError("FeatureType expects a model or queryset")
 
         self._fields = fields
+        self.display_field_name = display_field_name
         self._geometry_field_name = geometry_field_name
         self.name = name or self.model._meta.model_name
         self.title = title or self.model._meta.verbose_name
@@ -358,9 +403,13 @@ class FeatureType:
         self.other_crs = other_crs or []
         self.metadata_url = metadata_url
 
+        # Settings
+        self.show_name_field = show_name_field
+        self.xml_prefix = xml_prefix
+
         # Validate that the name doesn't require XML escaping.
         if html.escape(self.name) != self.name or " " in self.name or ":" in self.name:
-            raise ValueError(f"Invalid feature name for XML: <app:{self.name}>")
+            raise ValueError(f"Invalid feature name for XML: <{self.xml_name}>")
 
         # Auto-detect geometry fields (also fills geometry_field_name)
         self.geometry_fields = [
@@ -379,7 +428,12 @@ class FeatureType:
     @cached_property
     def xml_name(self):
         """Return the feature name with xml namespace prefix."""
-        return f"app:{self.name}"
+        return f"{self.xml_prefix}:{self.name}"
+
+    @cached_property
+    def supported_crs(self) -> List[CRS]:
+        """Return all spatial reference system ID's that this feature supports."""
+        return [self.crs] + self.other_crs
 
     @cached_property
     def geometry_field_names(self):
@@ -394,6 +448,14 @@ class FeatureType:
         else:
             return _get_model_fields(self.model, self._fields)
 
+    @cached_property
+    def display_field(self) -> Optional[models.Field]:
+        """Give access to the field that holds the string-representation."""
+        if not self.show_name_field or not self.display_field_name:
+            return None
+        else:
+            return self.model._meta.get_field(self.display_field_name)
+
     @cached_property
     def geometry_field(self) -> gis_models.GeometryField:
         """Give access to the Django field that holds the geometry."""
@@ -439,15 +501,16 @@ class FeatureType:
         when the database table is empty, or the custom queryset doesn't
         return any results.
         """
-        if self.geometry_field.srid != WGS84.srid:
-            geo_expression = Transform(self.geometry_field.name, WGS84.srid)
-        else:
-            geo_expression = self.geometry_field.name
+        geo_expression = conditional_transform(
+            self.geometry_field.name, self.geometry_field.srid, WGS84.srid
+        )
 
         bbox = self.get_queryset().aggregate(a=Extent(geo_expression))["a"]
         return BoundingBox(*bbox, crs=WGS84) if bbox else None
 
-    def get_envelope(self, instance, crs: CRS) -> Optional[BoundingBox]:
+    def get_envelope(
+        self, instance, crs: Optional[CRS] = None
+    ) -> Optional[BoundingBox]:
         """Get the bounding box for a single instance.
 
         This is only used for native Python rendering. When the database
@@ -466,16 +529,61 @@ class FeatureType:
         geometry = (
             geometries[0] if len(geometries) == 1 else reduce(operator.or_, geometries)
         )
-        if geometry.srid != crs.srid:
+        if crs is not None and geometry.srid != crs.srid:
             crs.apply_to(geometry)  # avoid clone
         return BoundingBox.from_geometry(geometry, crs=crs)
 
+    def get_display_value(self, instance: models.Model) -> str:
+        """Generate the display name value"""
+        if self.display_field_name:
+            return getattr(instance, self.display_field_name)
+        else:
+            return str(instance)
+
+    @property
+    def xsd_base_type(self) -> XsdAnyType:
+        """Return the base class for the :attr:`xsd_type`.
+
+        This builds an XsdComplexType element that represents
+        the contents of :attr:`XsdTypes.gmlAbstractFeatureType`.
+        By providing this as Complex Type, the filters can also resolve the
+        attributes of ``@gml:id``, ``<gml:name>`` and ``<gml:boundedBy>`` nodes.
+        """
+        pk_field = self.model._meta.pk
+
+        # Define <gml:boundedBy>
+        base_elements = [GmlBoundedByElement(feature_type=self)]
+        if self.show_name_field:
+            # Add <gml:name>
+            gml_name = GmlNameElement(
+                model_attribute=self.display_field_name,
+                source=self.display_field,
+                feature_type=self,
+            )
+            base_elements.insert(0, gml_name)
+
+        return XsdComplexType(
+            prefix="gml",
+            name="AbstractFeatureType",
+            elements=base_elements,
+            attributes=[
+                # Add gml:id attribute definition so it can be resolved in xpath
+                GmlIdAttribute(
+                    type_name=self.name,
+                    source=pk_field,
+                    model_attribute=pk_field.name,
+                )
+            ],
+            base=XsdTypes.gmlAbstractGMLType,
+        )
+
     @cached_property
     def xsd_type(self) -> XsdComplexType:
         """Return the definition of this feature as an XSD Complex Type."""
-        return XsdComplexType(
+        return self.xsd_type_class(
             name=f"{self.name.title()}Type",
             elements=[field.xsd_element for field in self.fields],
+            base=self.xsd_base_type,
             source=self.model,
         )
 
@@ -483,11 +591,11 @@ class FeatureType:
         """Resolve the element, and the matching object.
         Internally, this method caches results.
         """
-        path = self._cached_resolver(xpath)  # calls _inner_resolve_element
-        if path is None:
+        nodes = self._cached_resolver(xpath)  # calls _inner_resolve_element
+        if nodes is None:
             raise ExternalValueError(f"Field '{xpath}' does not exist.")
 
-        return XPathMatch(path, query=xpath)
+        return XPathMatch(self, nodes, query=xpath)
 
     def _inner_resolve_element(self, xpath: str):
         """Inner part of resolve_element() that is cached.
@@ -513,8 +621,8 @@ class FeatureType:
         for root_prefix in (
             f"{self.name}/",
             f"/{self.name}/",
-            f"app:{self.name}/",
-            f"/app:{self.name}/",
+            f"{self.xml_name}/",
+            f"/{self.xml_name}/",
         ):
             if xpath.startswith(root_prefix):
                 xpath = xpath[len(root_prefix) :]
