@@ -1,19 +1,20 @@
 import io
+import math
 from collections import defaultdict
-from typing import List, Set, Tuple, Type, cast
+from typing import List, Set, Tuple, Type
 
 from django.conf import settings
-from django.contrib.gis.db.models import GeometryField
-from django.contrib.gis.db.models.functions import Transform
 from django.db import models
 from django.db.models import Prefetch
 from django.http import HttpResponse, StreamingHttpResponse
 from django.utils.html import escape
 
+from gisserver import conf
+from gisserver.exceptions import InvalidParameterValue
 from gisserver.features import FeatureType
 from gisserver.geometries import CRS
 from gisserver.operations.base import WFSMethod
-from gisserver.types import XPathMatch, XsdComplexType, XsdElement
+from gisserver.types import XsdComplexType, XsdElement
 
 from .results import FeatureCollection
 
@@ -31,6 +32,9 @@ class OutputRenderer:
     #: Define the content type for rendering the output
     content_type = "application/octet-stream"
 
+    #: An optional content-disposition header to output
+    content_disposition = None
+
     def __init__(
         self,
         method: WFSMethod,
@@ -58,9 +62,14 @@ class OutputRenderer:
         self.app_xml_namespace = method.view.xml_namespace
 
     @classmethod
-    def decorate_collection(cls, collection: FeatureCollection, output_crs, **params):
+    def decorate_collection(
+        cls, collection: FeatureCollection, output_crs: CRS, **params
+    ):
         """Perform presentation-layer logic enhancements on the queryset."""
         for sub_collection in collection.results:
+            # Validate the presentation-level parameters for this feature:
+            cls.validate(sub_collection.feature_type, **params)
+
             queryset = cls.decorate_queryset(
                 sub_collection.feature_type,
                 sub_collection.queryset,
@@ -70,6 +79,20 @@ class OutputRenderer:
             if queryset is not None:
                 sub_collection.queryset = queryset
 
+    @classmethod
+    def validate(cls, feature_type: FeatureType, **params):
+        """Validate the presentation parameters"""
+        crs = params["srsName"]
+        if (
+            conf.GISSERVER_SUPPORTED_CRS_ONLY
+            and crs is not None
+            and crs not in feature_type.supported_crs
+        ):
+            raise InvalidParameterValue(
+                "srsName",
+                f"Feature '{feature_type.name}' does not support SRID {crs.srid}.",
+            )
+
     @classmethod
     def decorate_queryset(
         cls,
@@ -152,13 +175,46 @@ class OutputRenderer:
         """Render the output as streaming response."""
         stream = self.render_stream()
         if isinstance(stream, (str, bytes)):
-            return HttpResponse(content=stream, content_type=self.content_type)
+            response = HttpResponse(content=stream, content_type=self.content_type)
         else:
             stream = self._trap_exceptions(stream)
-            return StreamingHttpResponse(
-                streaming_content=stream, content_type=self.content_type,
+            response = StreamingHttpResponse(
+                streaming_content=stream,
+                content_type=self.content_type,
             )
 
+        for name, value in self.get_headers().items():
+            response[name] = value
+        return response
+
+    def get_headers(self):
+        """Return the response headers"""
+        if self.content_disposition:
+            # Offer a common quick content-disposition logic that works for all possible queries.
+            sub_collection = self.collection.results[0]
+            if sub_collection.stop == math.inf:
+                if sub_collection.start:
+                    page = f"{sub_collection.start}-end"
+                else:
+                    page = "all"
+            elif sub_collection.stop:
+                page = f"{sub_collection.start}-{sub_collection.stop - 1}"
+            else:
+                page = "results"
+
+            return {
+                "Content-Disposition": self.content_disposition.format(
+                    typenames="+".join(
+                        sub.feature_type.name for sub in self.collection.results
+                    ),
+                    page=page,
+                    date=self.collection.date.strftime("%Y-%m-%d %H.%M.%S%z"),
+                    timestamp=self.collection.timestamp,
+                )
+            }
+
+        return {}
+
     def _trap_exceptions(self, stream):
         """Decorate the generator to show exceptions"""
         try:
@@ -170,7 +226,12 @@ class OutputRenderer:
             raise
 
     def render_exception(self, exception: Exception):
-        """Render the exception in a format that fits with the output."""
+        """Inform the client that the stream processing was interrupted with an exception.
+        The exception can be rendered in the format fits with the output.
+
+        Purposefully, not much information is given, so avoid informing clients.
+        The actual exception is still raised and logged server-side.
+        """
         if settings.DEBUG:
             return f"<!-- {exception.__class__.__name__}: {exception} -->\n"
         else:
@@ -228,55 +289,3 @@ class StringBuffer(BaseBuffer):
 
     def __str__(self):
         return self.getvalue()
-
-
-def build_db_annotations(selects: dict, name_template: str, wrapper_func) -> dict:
-    """Utility to build annotations for all geometry fields for an XSD type.
-    This is used by various DB-optimized rendering methods.
-    """
-    return {
-        escape_xml_name(name, name_template): wrapper_func(target)
-        for name, target in selects.items()
-    }
-
-
-def get_db_annotation(instance: models.Model, name: str, name_template: str):
-    """Retrieve the value that an annotation has added to the model."""
-    # The "name" allows any XML-tag elements, escape the most obvious
-    return getattr(instance, escape_xml_name(name, name_template))
-
-
-def escape_xml_name(name: str, template="{name}") -> str:
-    """Escape an XML name to be used as annotation name."""
-    return template.format(name=name.replace(".", "_"))
-
-
-def get_db_geometry_selects(gml_elements: List[XsdElement], output_crs: CRS) -> dict:
-    """Utility to generate select clauses for the geometry fields of a type."""
-    return {
-        xsd_element.name: _get_db_geometry_target(xsd_element, output_crs)
-        for xsd_element in gml_elements
-        if xsd_element.source is not None
-    }
-
-
-def _get_db_geometry_target(xsd_element: XsdElement, output_crs: CRS):
-    """Wrap the selection of a geometry field in a CRS Transform if needed."""
-    field = cast(GeometryField, xsd_element.source)
-
-    target = xsd_element.orm_path
-    if field.srid != output_crs.srid:
-        target = Transform(target, output_crs.srid)
-
-    return target
-
-
-def get_db_geometry_target(xpath_match: XPathMatch, output_crs: CRS):
-    """Based on a resolved element, build the proper geometry field select clause."""
-    field = cast(GeometryField, xpath_match.child.source)
-
-    target = xpath_match.orm_path
-    if field.srid != output_crs.srid:
-        target = Transform(target, output_crs.srid)
-
-    return target
