@@ -4,29 +4,24 @@ from datetime import datetime
 from typing import List
 
 from django.conf import settings
-from django.contrib.gis.db.models.functions import GeoFunc
 from django.db import models
 from django.utils.timezone import utc
 
 from gisserver import conf
+from gisserver.db import (
+    AsEWKT,
+    build_db_annotations,
+    get_db_annotation,
+    get_db_geometry_selects,
+)
 from gisserver.features import FeatureType
 from gisserver.types import XsdComplexType, XsdElement
 from .base import (
     OutputRenderer,
     StringBuffer,
-    build_db_annotations,
-    get_db_annotation,
-    get_db_geometry_selects,
 )
 
 
-class _AsEWKT(GeoFunc):
-    """Generate EWKT in the database (PostGIS tested only at the moment)."""
-
-    name = "AsEWKT"
-    output_field = models.TextField()
-
-
 class CSVRenderer(OutputRenderer):
     """Fast CSV renderer, using a stream response.
 
@@ -34,7 +29,11 @@ class CSVRenderer(OutputRenderer):
     """
 
     content_type = "text/csv; charset=utf-8"
+    content_disposition = 'attachment; filename="{typenames} {page} {date}.csv"'
     max_page_size = conf.GISSERVER_CSV_MAX_PAGE_SIZE
+
+    #: The outputted CSV dialect. This can be a csv.Dialect subclass
+    #: or one of the registered names like: "unix", "excel", "excel-tab"
     dialect = "unix"
 
     @classmethod
@@ -113,7 +112,7 @@ class CSVRenderer(OutputRenderer):
         values = []
         append = values.append
         for field in fields:
-            if field.is_gml:
+            if field.is_geometry:
                 append(self.render_geometry(instance, field))
                 continue
 
@@ -149,11 +148,11 @@ class DBCSVRenderer(CSVRenderer):
         # Instead of reading the binary geometry data,
         # ask the database to generate EWKT data directly.
         geometries = get_db_geometry_selects(
-            feature_type.xsd_type.gml_elements, output_crs
+            feature_type.xsd_type.geometry_elements, output_crs
         )
         if geometries:
             queryset = queryset.defer(*geometries.keys()).annotate(
-                **build_db_annotations(geometries, "_as_ewkt_{name}", _AsEWKT)
+                **build_db_annotations(geometries, "_as_ewkt_{name}", AsEWKT)
             )
 
         return queryset
