@@ -9,11 +9,13 @@ from django.contrib.gis.db.models.functions import AsGeoJSON
 from django.db import models
 from django.utils.functional import Promise
 from django.utils.timezone import utc
+
 from gisserver import conf
+from gisserver.db import get_db_geometry_target
 from gisserver.features import FeatureType
 from gisserver.types import XsdComplexType
 
-from .base import BytesBuffer, OutputRenderer, get_db_geometry_target
+from .base import BytesBuffer, OutputRenderer
 
 
 def _json_default(obj):
@@ -35,6 +37,7 @@ class GeoJsonRenderer(OutputRenderer):
     """
 
     content_type = "application/json; charset=utf-8"
+    content_disposition = 'inline; filename="{typenames} {page} {date}.geojson"'
     max_page_size = conf.GISSERVER_GEOJSON_MAX_PAGE_SIZE
 
     @classmethod
@@ -56,7 +59,7 @@ class GeoJsonRenderer(OutputRenderer):
                 defer_related = [
                     f"{e.orm_path}__{child.orm_path}"
                     for e in relations
-                    for child in e.type.gml_elements
+                    for child in e.type.geometry_elements
                 ]
                 queryset = queryset.select_related(*related)
                 if defer_related:
@@ -125,17 +128,23 @@ class GeoJsonRenderer(OutputRenderer):
         # Get all instance attributes:
         properties = self.get_properties(feature_type.xsd_type, instance)
 
+        if feature_type.show_name_field:
+            name = feature_type.get_display_value(instance)
+            geometry_name = b'"geometry_name":%b,' % orjson.dumps(name)
+        else:
+            geometry_name = b""
+
         return (
             b"    {"
-            b'"type": "Feature", '
-            b'"id":%b, '
-            b'"geometry_name":%b,'
+            b'"type":"Feature",'
+            b'"id":%b,'
+            b"%b"
             b'"geometry":%b,'
             b'"properties":%b'
             b"}"
         ) % (
             orjson.dumps(f"{feature_type.name}.{instance.pk}"),
-            orjson.dumps(str(instance)),
+            geometry_name,
             self.render_geometry(feature_type, instance),
             orjson.dumps(properties, default=_json_default),
         )
@@ -220,7 +229,7 @@ class GeoJsonRenderer(OutputRenderer):
         """
         props = {}
         for xsd_element in xsd_type.elements:
-            if not xsd_element.is_gml:
+            if not xsd_element.is_geometry:
                 value = xsd_element.get_value(instance)
                 if xsd_element.type.is_complex_type:
                     # Nested object data
