@@ -6,16 +6,23 @@ as it's quite a performance improvement to just use html.escape().
 import itertools
 import re
 from datetime import date, datetime, time
-from functools import reduce
 from html import escape
 from typing import List, Optional, cast
 
 from django.contrib.gis import geos
-from django.contrib.gis.db.models.functions import AsGML, Transform, Union
-from django.db import connections, models
+from django.db import models
 from django.http import HttpResponse
 from django.utils.timezone import utc
 
+from gisserver.db import (
+    AsGML,
+    conditional_transform,
+    get_geometries_union,
+    build_db_annotations,
+    get_db_annotation,
+    get_db_geometry_selects,
+    get_db_geometry_target,
+)
 from gisserver.exceptions import NotFound
 from gisserver.features import FeatureType
 from gisserver.geometries import CRS
@@ -25,10 +32,6 @@ from gisserver.types import XsdComplexType, XsdElement
 from .base import (
     OutputRenderer,
     StringBuffer,
-    build_db_annotations,
-    get_db_annotation,
-    get_db_geometry_selects,
-    get_db_geometry_target,
 )
 
 GML_RENDER_FUNCTIONS = {}
@@ -42,33 +45,6 @@ def default_if_none(value, default):
         return value
 
 
-class _AsGML(AsGML):
-    name = "AsGML"
-
-    def __init__(self, expression, version=3, precision=14, envelope=False, **extra):
-        # Note that Django's AsGml, version=2, precision=8
-        # the options is postgres-only.
-        super().__init__(expression, version, precision, **extra)
-        self.envelope = envelope
-
-    def as_postgresql(self, compiler, connection, **extra_context):
-        # Fill options parameter (https://postgis.net/docs/ST_AsGML.html)
-        options = 33 if self.envelope else 1  # 32 = bbox, 1 = long CRS urn
-        template = f"%(function)s(%(expressions)s, {options})"
-        return self.as_sql(compiler, connection, template=template, **extra_context)
-
-
-class ST_Union(Union):
-    name = "Union"
-    arity = None
-
-    def as_postgresql(self, compiler, connection, **extra_context):
-        # PostgreSQL can handle ST_Union(ARRAY(field names)), other databases don't.
-        if len(self.source_expressions) > 2:
-            extra_context["template"] = f"%(function)s(ARRAY(%(expressions)s))"
-        return self.as_sql(compiler, connection, **extra_context)
-
-
 def register_geos_type(geos_type):
     def _inc(func):
         GML_RENDER_FUNCTIONS[geos_type] = func
@@ -90,13 +66,35 @@ class GML32Renderer(OutputRenderer):
         if isinstance(self.source_query, GetFeatureById):
             # WFS spec requires that GetFeatureById output only returns the contents.
             # The streaming response is avoided here, to allow returning a 404.
-            return HttpResponse(
-                content=self.render_xml_standalone(), content_type=self.content_type,
-            )
+            return self.get_standalone_response()
         else:
             # Use default streaming response, with render_stream()
             return super().get_response()
 
+    def get_standalone_response(self):
+        """Render a standalone item, for GetFeatureById"""
+        sub_collection = self.collection.results[0]
+        self.start_collection(sub_collection)
+        instance = sub_collection.first()
+        if instance is None:
+            raise NotFound("Feature not found.")
+
+        body = self.render_wfs_member_contents(
+            feature_type=sub_collection.feature_type,
+            instance=instance,
+            extra_xmlns=self.render_xmlns_standalone(),
+        ).lstrip(" ")
+
+        if body.startswith("<"):
+            return HttpResponse(
+                content=f'<?xml version="1.0" encoding="UTF-8"?>\n{body.lstrip()}',
+                content_type=self.content_type,
+            )
+        else:
+            # Best guess for GetFeatureById combined with
+            # GetPropertyValue&VALUEREFERENCE=@gml:id
+            return HttpResponse(body, content_type="text/plain")
+
     def render_xmlns(self):
         """Generate the xmlns block that the documente needs"""
         xsd_typenames = ",".join(
@@ -121,7 +119,7 @@ class GML32Renderer(OutputRenderer):
         )
 
     def render_xmlns_standalone(self):
-        """Generate the xmlns block that the documente needs"""
+        """Generate the xmlns block that the document needs"""
         # xsi is needed for "xsi:nil="true"' attributes.
         return (
             f' xmlns:app="{escape(self.app_xml_namespace)}"'
@@ -129,24 +127,6 @@ class GML32Renderer(OutputRenderer):
             ' xmlns:gml="http://www.opengis.net/gml/3.2"'
         )
 
-    def render_xml_standalone(self):
-        """Render a standalone item, for GetFeatureById"""
-        sub_collection = self.collection.results[0]
-        instance = sub_collection.first()
-        if instance is None:
-            raise NotFound("Feature not found.")
-
-        output = StringBuffer()
-        output.write('<?xml version="1.0" encoding="UTF-8"?>\n')
-        output.write(
-            self.render_wfs_member(
-                feature_type=sub_collection.feature_type,
-                instance=instance,
-                extra_xmlns=self.render_xmlns_standalone(),
-            )
-        )
-        return output.getvalue()
-
     def render_stream(self):
         """Render the XML as streaming content.
         This renders the standard <wfs:FeatureCollection> / <wfs:ValueCollection>
@@ -178,6 +158,7 @@ class GML32Renderer(OutputRenderer):
             has_multiple_collections = len(collection.results) > 1
 
             for sub_collection in collection.results:
+                self.start_collection(sub_collection)
                 if has_multiple_collections:
                     output.write(
                         f"  <wfs:member>\n"
@@ -188,11 +169,9 @@ class GML32Renderer(OutputRenderer):
                     )
 
                 for instance in sub_collection:
-                    output.write("  <wfs:member>\n")
                     output.write(
                         self.render_wfs_member(sub_collection.feature_type, instance)
                     )
-                    output.write("  </wfs:member>\n")
 
                     # Only perform a 'yield' every once in a while,
                     # as it goes back-and-forth for writing it to the client.
@@ -208,36 +187,58 @@ class GML32Renderer(OutputRenderer):
         output.write(f"</wfs:{self.xml_collection_tag}>\n")
         yield output.getvalue()
 
+    def start_collection(self, sub_collection):
+        """Hook to allow initialization per feature type"""
+        pass
+
     def render_wfs_member(
         self, feature_type: FeatureType, instance: models.Model, extra_xmlns=""
+    ):
+        """Write the full <wfs:member> block."""
+        body = self.render_wfs_member_contents(
+            feature_type, instance, extra_xmlns=extra_xmlns
+        )
+        return f"  <wfs:member>\n{body}  </wfs:member>\n"
+
+    def render_wfs_member_contents(
+        self, feature_type: FeatureType, instance: models.Model, extra_xmlns=""
     ) -> str:
         """Write the contents of the object value.
 
         This output is typically wrapped in <wfs:member> tags
         unless it's used for a GetPropertyById response.
         """
-        self.gml_seq = 0  # need to increment this between render_xml_field cals
+        self.gml_seq = 0  # need to increment this between render_xml_field calls
+
+        # Write <app:FeatureTypeName> start node
+        pk = escape(str(instance.pk))
         output = StringBuffer()
         output.write(
-            '    <app:{name} gml:id="{name}.{pk}"{extra_xmlns}>\n'
-            "      <gml:name>{display}</gml:name>\n".format(
-                name=feature_type.name,
-                pk=escape(str(instance.pk)),
-                display=escape(str(instance)),
-                extra_xmlns=extra_xmlns,
-            )
+            f'    <{feature_type.xml_name} gml:id="{feature_type.name}.{pk}"{extra_xmlns}>\n'
         )
 
-        # Add <gml:boundedBy>
-        gml = self.render_bounds(feature_type, instance)
-        if gml is not None:
-            output.write(gml)
+        # Add all base class members, in their correct ordering
+        # By having these as XsdElement objects instead of hard-coded writes,
+        # the query/filter logic also works for these elements.
+        if feature_type.xsd_type.base.is_complex_type:
+            for xsd_element in feature_type.xsd_type.base.elements:
+                if xsd_element.xml_name == "gml:boundedBy":
+                    # Special case for <gml:boundedBy>, so it will render with
+                    # the output CRS and can be overwritten with DB-rendered GML.
+                    gml = self.render_bounds(feature_type, instance)
+                    if gml is not None:
+                        output.write(gml)
+                else:
+                    # e.g. <gml:name>
+                    output.write(
+                        self.render_element(feature_type, xsd_element, instance)
+                    )
 
         # Add all members
         for xsd_element in feature_type.xsd_type.elements:
             output.write(self.render_element(feature_type, xsd_element, instance))
 
-        output.write(f"    </app:{feature_type.name}>\n")
+        output.write(f"    </{feature_type.xml_name}>\n")
         return output.getvalue()
 
     def render_bounds(self, feature_type, instance) -> Optional[str]:
@@ -258,12 +259,12 @@ class GML32Renderer(OutputRenderer):
     ):
         """Rendering of a single field."""
         value = xsd_element.get_value(instance)
-        if xsd_element.is_gml and value is not None:
+        if xsd_element.is_geometry and value is not None:
             # None check happens here to avoid incrementing for none values
             self.gml_seq += 1
             return self.render_gml_field(
                 feature_type,
-                xsd_element.name,
+                xsd_element,
                 value,
                 gml_id=self.get_gml_id(feature_type, instance.pk, seq=self.gml_seq),
             )
@@ -274,19 +275,19 @@ class GML32Renderer(OutputRenderer):
         self, feature_type: FeatureType, xsd_element: XsdElement, value, extra_xmlns=""
     ) -> str:
         """Write the value of a single field."""
-        name = xsd_element.name
+        xml_name = xsd_element.xml_name
         if value is None:
-            return f'      <app:{name} xsi:nil="true"{extra_xmlns} />\n'
+            return f'      <{xml_name} xsi:nil="true"{extra_xmlns} />\n'
         elif xsd_element.type.is_complex_type:
             # Expanded foreign relation / dictionary
             xsd_type = cast(XsdComplexType, xsd_element.type)
             output = StringBuffer()
-            output.write(f"      <app:{name}>\n")
+            output.write(f"      <{xml_name}>\n")
             for sub_element in xsd_type.elements:
                 output.write(
                     self.render_element(feature_type, sub_element, instance=value)
                 )
-            output.write(f"      </app:{name}>\n")
+            output.write(f"      </{xml_name}>\n")
             return output.getvalue()
         elif isinstance(value, datetime):
             value = value.astimezone(utc).isoformat()
@@ -297,14 +298,20 @@ class GML32Renderer(OutputRenderer):
         else:
             value = escape(str(value))
 
-        return f"      <app:{name}{extra_xmlns}>{value}</app:{name}>\n"
+        return f"      <{xml_name}{extra_xmlns}>{value}</{xml_name}>\n"
 
     def render_gml_field(
-        self, feature_type: FeatureType, name: str, value, gml_id, extra_xmlns=""
+        self,
+        feature_type: FeatureType,
+        xsd_element: XsdElement,
+        value,
+        gml_id,
+        extra_xmlns="",
     ) -> str:
         """Write the value of an GML tag"""
+        xml_name = xsd_element.xml_name
         gml = self.render_gml_value(value, gml_id=gml_id)
-        return f"      <app:{name}{extra_xmlns}>{gml}</app:{name}>\n"
+        return f"      <{xml_name}{extra_xmlns}>{gml}</{xml_name}>\n"
 
     def render_gml_value(
         self, value: geos.GEOSGeometry, gml_id: str, extra_xmlns=""
@@ -420,13 +427,13 @@ class DBGML32Renderer(GML32Renderer):
         )
 
         geometries = get_db_geometry_selects(
-            feature_type.xsd_type.gml_elements, output_crs
+            feature_type.xsd_type.geometry_elements, output_crs
         )
         return queryset.defer(*geometries.keys()).annotate(
             _as_envelope_gml=cls.get_db_envelope_as_gml(
                 feature_type, queryset, output_crs
             ),
-            **build_db_annotations(geometries, "_as_gml_{name}", _AsGML),
+            **build_db_annotations(geometries, "_as_gml_{name}", AsGML),
         )
 
     @classmethod
@@ -445,19 +452,19 @@ class DBGML32Renderer(GML32Renderer):
         # Find which fields are GML elements
         gml_elements = []
         for e in xsd_elements:
-            if e.is_gml:
+            if e.is_geometry:
                 # Prefetching a flattened relation
                 gml_elements.append(e)
             elif e.type.is_complex_type:
                 # Prefetching a complex type
                 xsd_type: XsdComplexType = cast(XsdComplexType, e.type)
-                gml_elements.extend(xsd_type.gml_elements)
+                gml_elements.extend(xsd_type.geometry_elements)
 
         geometries = get_db_geometry_selects(gml_elements, output_crs)
         fields = set(fields) - set(geometries.keys())
         if geometries:
             return model.objects.only("pk", *fields).annotate(
-                **build_db_annotations(geometries, "_as_gml_{name}", _AsGML),
+                **build_db_annotations(geometries, "_as_gml_{name}", AsGML),
             )
 
     @classmethod
@@ -467,42 +474,31 @@ class DBGML32Renderer(GML32Renderer):
         This also avoids offloads the geometry union calculation to the DB.
         """
         geo_fields_union = cls._get_geometries_union(feature_type, queryset, output_crs)
-        return _AsGML(geo_fields_union, envelope=True)
+        return AsGML(geo_fields_union, envelope=True)
 
     @classmethod
     def _get_geometries_union(cls, feature_type: FeatureType, queryset, output_crs):
         """Combine all geometries of the model in a single SQL function."""
-        field_names = feature_type.geometry_field_names
-        if len(field_names) == 1:
-            union = next(iter(field_names))  # fastest in set data type
-        elif len(field_names) == 2:
-            union = Union(*field_names)
-        elif connections[queryset.db].vendor == "postgresql":
-            # postgres can handle multiple field names
-            union = ST_Union(field_names)
-        else:
-            # other databases do Union(Union(1, 2), 3)
-            union = reduce(Union, field_names)
-
-        if feature_type.geometry_field.srid != output_crs.srid:
-            union = Transform(union, output_crs.srid)
-
-        return union
+        return conditional_transform(
+            get_geometries_union(feature_type.geometry_field_names, using=queryset.db),
+            feature_type.geometry_field.srid,
+            output_srid=output_crs.srid,
+        )
 
     def render_element(
         self, feature_type, xsd_element: XsdElement, instance: models.Model
     ):
-        if xsd_element.is_gml:
+        if xsd_element.is_geometry:
             # Optimized path, pre-rendered GML
             value = get_db_annotation(instance, xsd_element.name, "_as_gml_{name}")
             if value is None:
                 # Avoid incrementing gml_seq
-                return f'      <app:{xsd_element.name} xsi:nil="true" />\n'
+                return f'      <{xsd_element.xml_name} xsi:nil="true" />\n'
 
             self.gml_seq += 1
             return self.render_db_gml_field(
                 feature_type,
-                xsd_element.name,
+                xsd_element,
                 value,
                 gml_id=self.get_gml_id(feature_type, instance.pk, seq=self.gml_seq),
             )
@@ -510,11 +506,17 @@ class DBGML32Renderer(GML32Renderer):
             return super().render_element(feature_type, xsd_element, instance)
 
     def render_db_gml_field(
-        self, feature_type: FeatureType, name: str, value, gml_id, extra_xmlns=""
+        self,
+        feature_type: FeatureType,
+        xsd_element: XsdElement,
+        value,
+        gml_id,
+        extra_xmlns="",
     ) -> str:
         """Write the value of an GML tag"""
+        xml_name = xsd_element.xml_name
         if value is None:
-            return f'      <app:{name} xsi:nil="true"{extra_xmlns} />\n'
+            return f'      <{xml_name} xsi:nil="true"{extra_xmlns} />\n'
 
         # Write the gml:id inside the first tag
         pos = value.find(">")
@@ -525,7 +527,7 @@ class DBGML32Renderer(GML32Renderer):
             first_tag += f' gml:id="{escape(gml_id)}"'
 
         gml = first_tag + value[pos:]
-        return f"      <app:{name}{extra_xmlns}>{gml}</app:{name}>\n"
+        return f"      <{xml_name}{extra_xmlns}>{gml}</{xml_name}>\n"
 
     def render_bounds(self, feature_type, instance):
         """Generate the <gml:boundedBy> from DB prerendering."""
@@ -543,6 +545,7 @@ class GML32ValueRenderer(GML32Renderer):
     def __init__(self, *args, value_reference: ValueReference, **kwargs):
         self.value_reference = value_reference
         super().__init__(*args, **kwargs)
+        self.xsd_node = None
 
     @classmethod
     def decorate_queryset(
@@ -555,26 +558,51 @@ class GML32ValueRenderer(GML32Renderer):
         # Don't optimize queryset, it only retrieves one value
         return queryset
 
+    def start_collection(self, sub_collection):
+        # Resolve which XsdNode is being rendered
+        match = sub_collection.feature_type.resolve_element(self.value_reference.xpath)
+        self.xsd_node = match.child
+
     def render_wfs_member(
         self, feature_type: FeatureType, instance: dict, extra_xmlns=""
+    ):
+        """Overwritten to handle attribute support."""
+        if self.xsd_node.is_attribute:
+            # Attributes are rendered without any spaces.
+            # The format_value() is needed for @gml:id
+            body = self.xsd_node.format_value(instance["member"])
+            return f"  <wfs:member>{body}</wfs:member>\n"
+        else:
+            body = self.render_wfs_member_contents(feature_type, instance)
+            return f"  <wfs:member>\n{body}  </wfs:member>\n"
+
+    def render_wfs_member_contents(
+        self, feature_type: FeatureType, instance: dict, extra_xmlns=""
     ) -> str:
         """Write the XML for a single object."""
         value = instance["member"]
-        if isinstance(value, geos.GEOSGeometry):
+        if self.xsd_node.is_geometry:
             gml_id = self.get_gml_id(feature_type, instance["pk"], seq=1)
             return self.render_gml_field(
                 feature_type,
-                name=self.value_reference.element_name,
+                self.xsd_node,
                 value=value,
                 gml_id=gml_id,
                 extra_xmlns=extra_xmlns,
             )
         else:
             # The xsd_element is needed so render_xml_field() can render complex types.
-            xsd_element = feature_type.resolve_element(self.value_reference.xpath).child
-            return self.render_xml_field(
-                feature_type, xsd_element, value, extra_xmlns=extra_xmlns
-            )
+            value = self.xsd_node.format_value(value)  # needed for @gml:id
+            if self.xsd_node.is_attribute:
+                # For GetFeatureById, allow to return raw values
+                return str(value)
+            else:
+                return self.render_xml_field(
+                    feature_type,
+                    cast(XsdElement, self.xsd_node),
+                    value,
+                    extra_xmlns=extra_xmlns,
+                )
 
 
 class DBGML32ValueRenderer(DBGML32Renderer, GML32ValueRenderer):
@@ -587,10 +615,10 @@ class DBGML32ValueRenderer(DBGML32Renderer, GML32ValueRenderer):
         """Update the queryset to let the database render the GML output."""
         value_reference = params["valueReference"]
         match = feature_type.resolve_element(value_reference.xpath)
-        if match.child.is_gml:
+        if match.child.is_geometry:
             # Add 'gml_member' to point to the pre-rendered GML version.
             return queryset.values(
-                "pk", gml_member=_AsGML(get_db_geometry_target(match, output_crs)),
+                "pk", gml_member=AsGML(get_db_geometry_target(match, output_crs))
             )
         else:
             return queryset
@@ -601,12 +629,13 @@ class DBGML32ValueRenderer(DBGML32Renderer, GML32ValueRenderer):
         """Write the XML for a single object."""
         if "gml_member" in instance:
             gml_id = self.get_gml_id(feature_type, instance["pk"], seq=1)
-            return self.render_db_gml_field(
+            body = self.render_db_gml_field(
                 feature_type,
-                self.value_reference.element_name,
+                self.xsd_node,
                 instance["gml_member"],
                 gml_id=gml_id,
             )
+            return f"  <wfs:member>\n{body}  </wfs:member>\n"
         else:
             return super().render_wfs_member(
                 feature_type, instance, extra_xmlns=extra_xmlns
