@@ -11,23 +11,29 @@ from typing import List, Optional, Tuple, Union
 from xml.etree.ElementTree import Element
 
 from django.contrib.gis.geos import GEOSGeometry
-from django.db.models import F, Func, Q, Value
+from django.db.models import Func, Q, Value
 from django.db.models.expressions import Combinable
 
-from gisserver.parsers.base import FES20, BaseNode, TagNameEnum, tag_registry
+from gisserver.exceptions import ExternalParsingError
+from gisserver.parsers.base import BaseNode, TagNameEnum, tag_registry
 from gisserver.parsers.fes20.functions import function_registry
-from gisserver.parsers.utils import (
-    auto_cast,
-    expect_tag,
-    get_attribute,
-    xsd_cast,
+from gisserver.parsers.gml import (
+    GM_Envelope,
+    GM_Object,
+    TM_Object,
+    is_gml_element,
+    parse_gml_node,
 )
+from gisserver.parsers.utils import auto_cast, expect_tag, get_attribute
+from gisserver.types import ORMPath, FES20, XsdTypes, split_xml_name
 
 NoneType = type(None)
-
 RhsTypes = Union[
     Combinable, Func, Q, GEOSGeometry, bool, int, str, date, datetime, tuple
 ]
+ParsedValue = Union[
+    int, str, date, D, datetime, GM_Object, GM_Envelope, TM_Object, NoneType
+]
 
 OUTPUT_FIELDS = {
     bool: models.BooleanField(),
@@ -82,28 +88,53 @@ class Expression(BaseNode):
 class Literal(Expression):
     """The <fes:Literal> element that holds a literal value"""
 
-    raw_value: Optional[str]
-    type: Optional[str] = None
+    # The XSD definition even defines a sequence of xsd:any as possible member!
+    raw_value: Union[NoneType, str, GM_Object, GM_Envelope, TM_Object]
+    raw_type: Optional[str] = None
 
     def __str__(self):
         return self.value
 
     @cached_property
-    def value(
-        self,
-    ) -> Union[int, str, date, D, datetime, NoneType]:  # officially <xsd:any>
+    def value(self) -> ParsedValue:  # officially <xsd:any>
         """Access the value of the element, casted to the appropriate data type."""
-        if self.type:
+        if not isinstance(self.raw_value, str):
+            return self.raw_value  # GML element or None
+        elif self.type:
             # Cast the value based on the given xsd:QName
-            return xsd_cast(self.raw_value, self.type)
+            return self.type.to_python(self.raw_value)
         else:
             # Make sure gt / datetime comparisons work out of the box.
             return auto_cast(self.raw_value)
 
+    @cached_property
+    def type(self) -> Optional[XsdTypes]:
+        if not self.raw_type:
+            return None
+
+        xmlns, localname = split_xml_name(self.raw_type)
+        if xmlns == "gml":
+            return XsdTypes(self.raw_type)
+        else:
+            # No idea what XMLSchema was prefixed as (could be ns0 instead of xs:)
+            return XsdTypes(localname)
+
     @classmethod
-    @expect_tag(FES20, "Literal", leaf=True)
+    @expect_tag(FES20, "Literal")
     def from_xml(cls, element: Element):
-        return cls(raw_value=element.text, type=element.get("type"))
+        children = len(element)
+        if not children:
+            # Common case: value is raw text
+            raw_value = element.text
+        elif children == 1 and is_gml_element(element[0]):
+            # Possible: a <gml:Envelope> element to compare something against.
+            raw_value = parse_gml_node(element[0])
+        else:
+            raise ExternalParsingError(
+                f"Unsupported child element for <Literal> element: {element[0].tag}."
+            )
+
+        return cls(raw_value=raw_value, raw_type=element.get("type"))
 
     def build_lhs(self, compiler) -> str:
         """Alias the value when it's used in the left-hand-side.
@@ -119,10 +150,16 @@ class Literal(Expression):
         # When the value is used a left-hand-side, Django needs to know the output type.
         return OUTPUT_FIELDS.get(type(self.value))
 
-    def build_rhs(self, compiler) -> Union[Combinable, Q, str]:
+    def build_rhs(self, compiler) -> Union[Combinable, Q, ParsedValue]:
         """Return the value when it's used in the right-hand-side"""
         return self.value
 
+    def bind_type(self, type: XsdTypes):
+        """Assign the expected type that this literal is compared against"""
+        if not self.raw_type:
+            self.__dict__["type"] = type
+            self.__dict__.pop("value", None)  # reset cached_property
+
 
 @dataclass
 @tag_registry.register("ValueReference")
@@ -147,25 +184,23 @@ class ValueReference(Expression):
 
     def build_lhs(self, compiler) -> str:
         """Optimized LHS: there is no need to alias a field lookup through an annotation."""
-        field, extra_q = self.parse_xpath(compiler.feature_type)
-        if extra_q:
-            compiler.add_extra_lookup(extra_q)
-        return field
+        match = self.parse_xpath(compiler.feature_type)
+        return match.build_lhs(compiler)
 
     def build_rhs(self, compiler) -> RhsTypes:
         """Return the value as F-expression"""
-        return F(self.build_lhs(compiler))
+        match = self.parse_xpath(compiler.feature_type)
+        return match.build_rhs(compiler)
 
-    def parse_xpath(self, feature_type=None) -> Tuple[str, Optional[Q]]:
+    def parse_xpath(self, feature_type=None) -> ORMPath:
         """Convert the XPath into a the required ORM query elements."""
         if feature_type is not None:
             # Can resolve against XSD paths, find the correct DB field name
-            match = feature_type.resolve_element(self.xpath)
-            return match.orm_path, match.orm_filters
+            return feature_type.resolve_element(self.xpath)
         else:
             # Only used by unit testing (when feature_type is not given).
             parts = [word.strip() for word in self.xpath.split("/")]
-            return "__".join(parts), None
+            return ORMPath(orm_path="__".join(parts), orm_filters=None)
 
     @cached_property
     def element_name(self):
