@@ -3,11 +3,11 @@ from xml.etree.ElementTree import Element, QName
 
 from defusedxml.ElementTree import ParseError, fromstring
 
-from gisserver.exceptions import ExternalValueError
-from gisserver.parsers.base import FES20, tag_registry
+from gisserver.exceptions import ExternalParsingError
+from gisserver.parsers.base import tag_registry
 from gisserver.parsers.utils import expect_tag
+from gisserver.types import FES20, GML32
 from . import expressions, identifiers, operators, query
-from ..gml import GML32
 
 FilterPredicates = Union[expressions.Function, operators.Operator]
 
@@ -43,13 +43,13 @@ class Filter:
                 # Allow KVP requests without a namespace
                 # Both geoserver and mapserver support this.
                 if "<Filter" in first_tag:
-                    text = f'{first_tag} xmlns="{FES20}" xmlns:gml="{GML32}">{text[end_first:]}'
+                    text = f'{first_tag} xmlns="{FES20}" xmlns:gml="{GML32}"{text[end_first:]}'
 
         try:
             root_element = fromstring(text)
         except ParseError as e:
             # Offer consistent results for callers to check for invalid data.
-            raise ExternalValueError(str(e)) from e
+            raise ExternalParsingError(str(e)) from e
         return Filter.from_xml(root_element, source=text)
 
     @classmethod
@@ -72,9 +72,9 @@ class Filter:
                 source=source,
             )
 
-    def compile_query(self, feature_type=None) -> query.CompiledQuery:
+    def compile_query(self, feature_type=None, using=None) -> query.CompiledQuery:
         """Collect the data to perform a Django ORM query."""
-        compiler = query.CompiledQuery(feature_type=feature_type)
+        compiler = query.CompiledQuery(feature_type=feature_type, using=using)
 
         # Function, Operator, IdList
         q_object = self.predicate.build_query(compiler)
