@@ -8,16 +8,18 @@ from dataclasses import dataclass, field
 from decimal import Decimal
 from enum import Enum
 from functools import reduce
-from typing import Any, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 from xml.etree.ElementTree import Element
 
 from django.contrib.gis import measure
 from django.db.models import Q
+from django.utils.functional import cached_property
 
-from gisserver.exceptions import ExternalValueError
+from gisserver.exceptions import ExternalParsingError, OperationProcessingFailed
 from gisserver.parsers import gml
-from gisserver.parsers.base import FES20, BaseNode, TagNameEnum, tag_registry
+from gisserver.parsers.base import BaseNode, TagNameEnum, tag_registry
 from gisserver.parsers.utils import expect_tag, get_attribute, get_child
+from gisserver.types import FES20
 from .identifiers import Id
 from .expressions import Expression, Literal, RhsTypes, ValueReference
 from .query import CompiledQuery
@@ -66,7 +68,7 @@ class BinaryComparisonName(TagNameEnum):
 class DistanceOperatorName(TagNameEnum):
     """XML tag names for distance operators"""
 
-    Beyond = "fes_not_dwithin"  # using __distance_gt=.. would be slower.
+    Beyond = "fes_beyond"  # using __distance_gt=.. would be slower.
     DWithin = "dwithin"  # ST_DWithin uses indices, distance_lte does not.
 
 
@@ -154,15 +156,35 @@ class IdOperator(Operator):
 
     id: List[Id]
 
+    @property
+    def type_names(self) -> List[str]:
+        """Provide a list of all type names accessed by this operator"""
+        return [
+            type_name for type_name in self.grouped_ids.keys() if type_name is not None
+        ]
+
+    @cached_property
+    def grouped_ids(self) -> Dict[str, List[Id]]:
+        # For itertools.groupby(), items have to be sorted first.
+        ids = sorted(self.id, key=operator.attrgetter("rid"))
+        return {
+            type_name: list(items)
+            for type_name, items in groupby(ids, key=operator.attrgetter("type_name"))
+        }
+
     def build_query(self, compiler):
         """Generate the ID lookup query.
 
         As these identifiers also reference the type name, no Q-object is
         returned. The lookups are directly added to the fes query object.
         """
-        # For itertools.groupby(), items need to be sorted first.
-        ids = sorted(self.id, key=operator.attrgetter("rid"))
-        for type_name, items in groupby(ids, key=operator.attrgetter("type_name")):
+        # When ResourceId has no type_name associated, this means that the
+        # parsing was invalid. Instead of raising an error, empty results are returned.
+        if len(self.grouped_ids) == 1 and self.grouped_ids.get(None):
+            compiler.mark_empty()
+            return
+
+        for type_name, items in self.grouped_ids.items():
             ids_subset = reduce(
                 operator.or_, [id.build_query(compiler=None) for id in items]
             )
@@ -173,6 +195,7 @@ class NonIdOperator(Operator):
     """Abstract base class, as defined by FES spec."""
 
     _source = None
+    allow_geometries = False
 
     def build_compare(
         self,
@@ -187,19 +210,11 @@ class NonIdOperator(Operator):
         """
         # lhs and rhs are allowed to be reversed. However, the SQL compiler
         # works much simpler when Django can predict the actual data type.
-        if isinstance(rhs, Literal) and isinstance(rhs, ValueReference):
+        if isinstance(lhs, Literal) and isinstance(rhs, ValueReference):
             lhs, rhs = rhs, lhs
 
-        if (
-            compiler.feature_type is not None
-            and isinstance(lhs, ValueReference)
-            and isinstance(rhs, Literal)
-        ):
-            # When a common case of value comparison is done, the inputs
-            # can be validated before the ORM query is constructed.
-            xsd_element = compiler.feature_type.resolve_element(lhs.xpath).child
-            tag = self._source if self._source is not None else None
-            xsd_element.validate_comparison(rhs.raw_value, lookup=lookup, tag=tag)
+        if compiler.feature_type is not None:
+            self.validate_comparison(compiler, lhs, lookup, rhs)
 
         lhs = lhs.build_lhs(compiler)
 
@@ -209,6 +224,34 @@ class NonIdOperator(Operator):
         result = Q(**{f"{lhs}__{lookup}": rhs})
         return compiler.apply_extra_lookups(result)
 
+    def validate_comparison(
+        self, compiler, lhs: Expression, lookup: str, rhs: Union[Expression, RhsTypes]
+    ):
+        """Validate whether a given comparison is even possible.
+        Where needed, the lhs/rhs are already ordered in a logical sequence.
+        """
+        if isinstance(lhs, ValueReference):
+            xsd_element = compiler.feature_type.resolve_element(lhs.xpath).child
+            tag = self._source if self._source is not None else None
+
+            # e.g. deny <PropertyIsLessThanOrEqualTo> against <gml:boundedBy>
+            if xsd_element.is_geometry and not self.allow_geometries:
+                raise OperationProcessingFailed(
+                    "filter",
+                    f"Operator '{tag}' does not support comparing"
+                    f" geometry properties: '{xsd_element.xml_name}'.",
+                    status_code=400,  # not HTTP 500 here. Spec allows both.
+                )
+
+            if isinstance(rhs, Literal):
+                # Since the element is resolved, inform the Literal how to parse the value.
+                # This avoids various validation errors along the path.
+                rhs.bind_type(xsd_element.type)
+
+                # When a common case of value comparison is done, the inputs
+                # can be validated before the ORM query is constructed.
+                xsd_element.validate_comparison(rhs.raw_value, lookup=lookup, tag=tag)
+
     def build_compare_between(
         self,
         compiler: CompiledQuery,
@@ -217,6 +260,10 @@ class NonIdOperator(Operator):
         rhs: Tuple[HasBuildRhs, HasBuildRhs],
     ) -> Q:
         """Use the value in comparison with 2 other values (e.g. between query)"""
+        if compiler.feature_type is not None:
+            self.validate_comparison(compiler, lhs, lookup, rhs[0])
+            self.validate_comparison(compiler, lhs, lookup, rhs[1])
+
         field_name = lhs.build_lhs(compiler)
         result = Q(
             **{
@@ -238,6 +285,8 @@ class SpatialOperator(NonIdOperator):
 class DistanceOperator(SpatialOperator):
     """Comparing the distance to a geometry."""
 
+    allow_geometries = True  # override static attribute
+
     valueReference: ValueReference
     operatorType: DistanceOperatorName
     geometry: gml.GM_Object
@@ -248,9 +297,11 @@ class DistanceOperator(SpatialOperator):
     def from_xml(cls, element: Element):
         geometries = gml.find_gml_nodes(element)
         if not geometries:
-            raise ExternalValueError(f"Missing gml element in <{element.tag}>")
+            raise ExternalParsingError(f"Missing gml element in <{element.tag}>")
         elif len(geometries) > 1:
-            raise ExternalValueError(f"Multiple gml elements found in <{element.tag}>")
+            raise ExternalParsingError(
+                f"Multiple gml elements found in <{element.tag}>"
+            )
 
         return cls(
             valueReference=ValueReference.from_xml(
@@ -276,6 +327,8 @@ class DistanceOperator(SpatialOperator):
 class BinarySpatialOperator(SpatialOperator):
     """A comparison of geometries using 2 values, e.g. A Within B."""
 
+    allow_geometries = True  # override static attribute
+
     operatorType: SpatialOperatorName
     operand1: Optional[ValueReference]
     operand2: SpatialDescription
@@ -290,14 +343,14 @@ class BinarySpatialOperator(SpatialOperator):
             geo = element[0]
         else:
             if len(element) != 2:
-                raise ExternalValueError(f"{element.tag} should have 2 operators")
+                raise ExternalParsingError(f"{element.tag} should have 2 operators")
             ref, geo = list(element)
 
         return cls(
             operatorType=operator_type,
             operand1=ValueReference.from_xml(ref) if ref is not None else None,
             operand2=tag_registry.from_child_xml(
-                geo, allowed_types=SpatialDescription.__args__,  # get_args() in 3.8
+                geo, allowed_types=SpatialDescription.__args__  # get_args() in 3.8
             ),
             _source=element.tag,
         )
@@ -308,7 +361,10 @@ class BinarySpatialOperator(SpatialOperator):
             operant1 = ValueReference(xpath=compiler.feature_type.geometry_field_name)
 
         return self.build_compare(
-            compiler, lhs=operant1, lookup=self.operatorType.value, rhs=self.operand2,
+            compiler,
+            lhs=operant1,
+            lookup=self.operatorType.value,
+            rhs=self.operand2,
         )
 
 
@@ -328,7 +384,7 @@ class TemporalOperator(NonIdOperator):
             operatorType=TemporalOperatorName.from_xml(element),
             operand1=ValueReference.from_xml(element[0]),
             operand2=tag_registry.from_child_xml(
-                element[1], allowed_types=TemporalOperand.__args__,  # get_args() in 3.8
+                element[1], allowed_types=TemporalOperand.__args__  # get_args() in 3.8
             ),
             _source=element.tag,
         )
@@ -444,7 +500,9 @@ class LikeOperator(ComparisonOperator):
 
             rhs = Literal(raw_value=value)
         else:
-            raise NotImplementedError()
+            raise ExternalParsingError(
+                f"Expected a literal value for the {self.tag} operator."
+            )
 
         # Use the FesLike lookup
         return self.build_compare(compiler, lhs=lhs, lookup="fes_like", rhs=rhs)
