@@ -3,7 +3,7 @@ from functools import reduce
 from typing import Dict, List, Optional, Union
 
 from django.contrib.gis.db.models.fields import BaseSpatialField
-from django.contrib.gis.db.models.lookups import DistanceLookupBase
+from django.contrib.gis.db.models.lookups import DWithinLookup
 from django.db import models
 from django.db.models import Q, QuerySet, lookups
 from django.db.models.expressions import Combinable
@@ -25,18 +25,21 @@ class CompiledQuery:
     def __init__(
         self,
         feature_type: Optional[FeatureType] = None,
+        using: Optional[str] = None,
         lookups: Optional[List[Q]] = None,
         typed_lookups: Optional[Dict[str, List[Q]]] = None,
         annotations: Optional[Dict[str, Union[Combinable, Q]]] = None,
     ):
         """The init method is typically used only in unit tests."""
         self.feature_type = feature_type
+        self.using = using
         self.lookups = lookups or []
         self.typed_lookups = typed_lookups or {}
         self.annotations = annotations or {}
         self.aliases = 0
         self.extra_lookups = []
         self.ordering = []
+        self.is_empty = False
 
     def add_annotation(self, value: Union[Combinable, Q]) -> str:
         """Create an named-alias for a function/Q object.
@@ -86,6 +89,10 @@ class CompiledQuery:
             self.extra_lookups.clear()
         return result
 
+    def mark_empty(self):
+        """Mark as returning no results."""
+        self.is_empty = True
+
     def filter_queryset(
         self, queryset: QuerySet, feature_type: FeatureType
     ) -> QuerySet:
@@ -94,6 +101,9 @@ class CompiledQuery:
         :param queryset: The queryset to filter.
         :param feature_type: The feature type that the queryset originated from.
         """
+        if self.is_empty:
+            return queryset.none()
+
         if self.extra_lookups:
             # Each time an expression node calls add_extra_lookup(),
             # the parent should have used apply_extra_lookups()
@@ -175,6 +185,19 @@ class FesNotEqualTo(lookups.Lookup):
 
 
 @BaseSpatialField.register_lookup
-class FesNotDWithinLookup(DistanceLookupBase):
-    lookup_name = "fes_not_dwithin"
+class FesBeyondLookup(DWithinLookup):
+    """Based on the FES 2.0.3 corrigendum:
+
+    DWithin(A,B,d) = Distance(A,B) < d
+    Beyond(A,B,d) = Distance(A,B) > d
+
+    See: https://docs.opengeospatial.org/is/09-026r2/09-026r2.html#61
+    """
+
+    lookup_name = "fes_beyond"
     sql_template = "NOT %(func)s(%(lhs)s, %(rhs)s, %(value)s)"
+
+    def get_rhs_op(self, connection, rhs):
+        # Allow the SQL $(func)s to be different from the ORM lookup name.
+        # This uses ST_DWithin() on PostGIS
+        return connection.ops.gis_operators["dwithin"]
