@@ -6,11 +6,11 @@ from typing import List, Union
 from xml.etree.ElementTree import Element
 
 from defusedxml.ElementTree import fromstring
-from gisserver.exceptions import ExternalValueError
+from gisserver.exceptions import ExternalParsingError
 from gisserver.parsers.base import tag_registry
 
 from .base import AbstractGeometry, GM_Envelope, GM_Object, TM_Object
-from .geometries import GML21, GML32, is_gml_element  # also do tag registration
+from .geometries import is_gml_element  # also do tag registration
 
 # All known root nodes as GML object:
 FES_GML_NODES = (GM_Object, GM_Envelope, TM_Object)
@@ -23,8 +23,6 @@ __all__ = [
     "parse_gml",
     "parse_gml_node",
     "find_gml_nodes",
-    "GML21",
-    "GML32",
 ]
 
 
@@ -37,7 +35,7 @@ def parse_gml(text: Union[str, bytes]) -> Union[FES_GML_NODES]:
 def parse_gml_node(element: Element) -> Union[FES_GML_NODES]:
     """Parse the element"""
     if not is_gml_element(element):
-        raise ExternalValueError(f"Expected GML namespace for {element.tag}")
+        raise ExternalParsingError(f"Expected GML namespace for {element.tag}")
 
     return tag_registry.from_child_xml(element, allowed_types=FES_GML_NODES)
 
