@@ -11,6 +11,7 @@ from dataclasses import dataclass
 from django.db.models import Q
 from typing import List, Optional
 
+from gisserver import conf
 from gisserver.exceptions import (
     InvalidParameterValue,
     MissingParameterValue,
@@ -59,7 +60,7 @@ class AdhocQuery(QueryExpression):
 
     # Officially part of the GetFeature/GetPropertyValue request object,
     # but included here for ease of query implementation.
-    resourceId: Optional[fes20.ResourceId] = None
+    resourceId: Optional[fes20.IdOperator] = None
 
     # GetPropertyValue:
     # In the WFS spec, this is only part of the operation/presentation.
@@ -91,15 +92,19 @@ class AdhocQuery(QueryExpression):
             # When ResourceId + typenames is defined, it should be a value from typenames
             # see WFS spec 7.9.2.4.1
             if params["typeNames"]:
-                raw_type_names = [
-                    feature_type.name for feature_type in params["typeNames"]
-                ]
-                if params["resourceID"].type_name not in raw_type_names:
-                    raise InvalidParameterValue(
-                        "resourceID",
-                        "When TYPENAMES and RESOURCEID are combined, "
-                        "the RESOURCEID type should be included in TYPENAMES.",
-                    )
+                id_type_names = params["resourceID"].type_names
+                if id_type_names:
+                    # Only test when the RESOURCEID has a typename.id format
+                    # Otherwise, this breaks the CITE RESOURCEID=test-UUID parameter.
+                    kvp_type_names = {
+                        feature_type.name for feature_type in params["typeNames"]
+                    }
+                    if not kvp_type_names.issuperset(id_type_names):
+                        raise InvalidParameterValue(
+                            "resourceID",
+                            "When TYPENAMES and RESOURCEID are combined, "
+                            "the RESOURCEID type should be included in TYPENAMES.",
+                        )
 
         return AdhocQuery(
             typeNames=params["typeNames"],
@@ -117,35 +122,52 @@ class AdhocQuery(QueryExpression):
 
         if self.resourceId:
             # Early validation whether the selected resourceID type exists.
-            feature_type = self.resolve_type_name(
-                self.resourceId.type_name, locator="resourceID"
-            )
+            feature_types = [
+                self.resolve_type_name(type_name, locator="resourceID")
+                for type_name in self.resourceId.type_names
+            ]
 
             # Also make the behavior consistent, always supply the type name.
             if not self.typeNames:
-                self.typeNames = [feature_type]
+                self.typeNames = feature_types
 
     def get_type_names(self):
         return self.typeNames
 
-    def compile_query(self, feature_type: FeatureType) -> fes20.CompiledQuery:
+    def compile_query(
+        self, feature_type: FeatureType, using=None
+    ) -> fes20.CompiledQuery:
         """Return our internal CompiledQuery object that can be applied to the queryset."""
         if self.filter:
             # Generate the internal query object from the <fes:Filter>
-            return self.filter.compile_query(feature_type)
+            return self.filter.compile_query(feature_type, using=using)
         else:
             # Generate the internal query object from the BBOX and sortBy args.
-            return self._compile_non_filter_query(feature_type)
+            return self._compile_non_filter_query(feature_type, using=using)
 
-    def _compile_non_filter_query(self, feature_type):
+    def _compile_non_filter_query(self, feature_type, using=None):
         """Generate the query based on the remaining parameters.
 
         This is slightly more efficient then generating the fes Filter object
         from these KVP parameters (which could also be done within the request method).
         """
-        compiler = fes20.CompiledQuery(feature_type=feature_type)
+        compiler = fes20.CompiledQuery(feature_type=feature_type, using=using)
 
         if self.bbox:
+            # Validate whether the provided SRID is supported.
+            # While PostGIS would support many more ID's,
+            # it would crash when an unsupported ID is given.
+            crs = self.bbox.crs
+            if (
+                conf.GISSERVER_SUPPORTED_CRS_ONLY
+                and crs is not None
+                and crs not in feature_type.supported_crs
+            ):
+                raise InvalidParameterValue(
+                    "bbox",
+                    f"Feature '{feature_type.name}' does not support SRID {crs.srid}.",
+                )
+
             # Using __within does not work with geometries
             # that only partially exist within the bbox
             lookup = operators.SpatialOperatorName.BBOX.value  # "intersects"
