@@ -5,7 +5,7 @@ from gisserver.exceptions import InvalidParameterValue
 from gisserver.features import FeatureType
 from gisserver.output import FeatureCollection, SimpleFeatureCollection
 from gisserver.parsers import fes20
-from gisserver.types import strip_namespace_prefix
+from gisserver.types import split_xml_name
 
 
 class QueryExpression:
@@ -41,20 +41,20 @@ class QueryExpression:
         for feature_type in self.get_type_names():
             feature_type.check_permissions(request)
 
-    def resolve_type_name(self, type_name, locator="") -> FeatureType:
+    def resolve_type_name(self, type_name, locator="typename") -> FeatureType:
         """Find the feature type for a given name.
         This is an utility that cusstom subclasses can use.
         """
         # Strip the namespace prefix. The Python ElementTree parser does
         # not expose the used namespace prefixes, so text-values can't be
         # mapped against it. As we expose just one namespace, just strip it.
-        type_name = strip_namespace_prefix(type_name)
+        xmlns, type_name = split_xml_name(type_name)
 
         try:
             return self.all_feature_types[type_name]
         except KeyError:
             raise InvalidParameterValue(
-                "typename", f"Typename '{type_name}' doesn't exist in this server."
+                locator, f"Typename '{type_name}' doesn't exist in this server."
             ) from None
 
     def get_hits(self) -> FeatureCollection:
@@ -111,7 +111,7 @@ class QueryExpression:
         queryset = feature_type.get_queryset()
 
         # Apply filters
-        compiler = self.compile_query(feature_type)
+        compiler = self.compile_query(feature_type, using=queryset.db)
 
         if self.value_reference is not None:
             if feature_type.resolve_element(self.value_reference.xpath) is None:
@@ -138,7 +138,9 @@ class QueryExpression:
             f"{self.__class__.__name__}.get_type_names() should be implemented."
         )
 
-    def compile_query(self, feature_type: FeatureType) -> fes20.CompiledQuery:
+    def compile_query(
+        self, feature_type: FeatureType, using=None
+    ) -> fes20.CompiledQuery:
         """Define the compiled query that filters the queryset.
 
         Subclasses need to define this method, unless
