@@ -72,7 +72,7 @@ class StoredQuery(QueryExpression):
     def extract_parameters(cls, KVP) -> Dict[str, str]:
         """Extract the arguments from the key-value-pair (=HTTP GET) request."""
         args = {}
-        for name, xsd_type in cls.meta.parameters.items():
+        for name, _xsd_type in cls.meta.parameters.items():
             try:
                 args[name] = KVP[name]
             except KeyError:
@@ -197,7 +197,9 @@ class GetFeatureById(StoredQuery):
 
         return collection
 
-    def compile_query(self, feature_type: FeatureType) -> fes20.CompiledQuery:
+    def compile_query(
+        self, feature_type: FeatureType, using=None
+    ) -> fes20.CompiledQuery:
         """Create the internal query object that will be applied to the queryset."""
         compiler = fes20.CompiledQuery(feature_type=feature_type)
         compiler.add_lookups(Q(pk=self.id), type_name=self.type_name)
