@@ -1,5 +1,6 @@
 """The view layer parses the request, and dispatches it to an operation."""
 import re
+from django.shortcuts import render
 from typing import List, Optional, Type
 
 from django.core.exceptions import (
@@ -36,6 +37,9 @@ class GISView(View):
     #: Default version to use
     version = "2.0.0"
 
+    #: Allow to set a default service, so the SERVICE parameter can be omitted.
+    default_service = None
+
     #: Supported versions
     accept_versions = ("2.0.0",)
 
@@ -45,6 +49,12 @@ class GISView(View):
     #: Metadata of the service:
     service_description: Optional[ServiceDescription] = None
 
+    #: Template to render a HTML welcome page for non-OGC requests.
+    index_template_name = None
+
+    #: Whether to render GET HTML pages
+    use_html_templates = True
+
     def dispatch(self, request, *args, **kwargs):
         """Render proper XML errors for exceptions on all request types."""
         try:
@@ -87,16 +97,90 @@ class GISView(View):
         """
         # Convert to WFS key-value-pair format.
         self.KVP = {key.upper(): value for key, value in request.GET.items()}
+
+        # Perform early version parsing. The detailed validation happens by the operation
+        # Parameter objects, but by performing an early check, templates can use that version.
+        version = self.KVP.get("VERSION")
+        if version and version in self.accept_versions:
+            self.set_version(version)
+
+        # Allow for an user-friendly opening page (hence the version check above)
+        if self.use_html_templates and self.is_index_request():
+            return self.render_index()
+
+        # Normal WFS
         wfs_method_cls = self.get_operation_class()
         return self.call_operation(wfs_method_cls)
 
+    def is_index_request(self):
+        """Tell whether to index page should be shown."""
+        # If none of the typical WFS parameters are given, and there aren't many other
+        # (misspelled?) parameters on the query string, this is considered to be an index page.
+        # Minimal request has 2 parameters (SERVICE=WFS&REQUEST=GetCapabilities).
+        # Some servers also allow a default of SERVICE, thus needing even less.
+        return (
+            len(self.KVP) < 2
+            and {
+                "REQUEST",
+                "SERVICE",
+                "VERSION",
+                "ACCEPTVERSIONS",
+            }.isdisjoint(self.KVP.keys())
+        )
+
+    def render_index(self):
+        """Render the index page."""
+        # Not using the whole TemplateResponseMixin config with configurable parameters.
+        # If this really needs more configuration, overriding is likely just as easy.
+        return render(
+            self.request, self.get_index_template_names(), self.get_index_context_data()
+        )
+
+    def get_index_context_data(self, **kwargs):
+        """Provide the context data for the index page."""
+        service = self.KVP.get("SERVICE", self.default_service)
+        root_url = self.request.build_absolute_uri()
+        return {
+            "view": self,
+            "service": service,
+            "root_url": root_url,
+            "connect_url": root_url,
+            "version": self.version,
+            "service_description": self.get_service_description(service),
+            "accept_versions": self.accept_versions,
+            "accept_operations": self.accept_operations,
+            **kwargs,
+        }
+
+    def get_index_template_names(self):
+        """Get the index page template name.
+        If no template is configured, some reasonable defaults are selected.
+        """
+        raw_service = self.KVP.get("SERVICE", self.default_service)
+        if raw_service and raw_service in self.accept_operations:
+            service = raw_service.lower()
+        else:
+            service = "default"
+
+        if self.index_template_name:
+            # Allow the same substitutions for a manually configured template.
+            return [
+                self.index_template_name.format(service=service, version=self.version)
+            ]
+        else:
+            return [
+                f"gisserver/{service}/{self.version}/index.html",
+                f"gisserver/{service}/index.html",
+                "gisserver/index.html",
+            ]
+
     def get_operation_class(self) -> Type[base.WFSMethod]:
         """Resolve the method that the client wants to call."""
         if not self.accept_operations:
             raise ImproperlyConfigured("View has no operations")
 
         # The service is always WFS
-        service = self._get_required_arg("SERVICE").upper()
+        service = self._get_required_arg("SERVICE", self.default_service).upper()
         try:
             operations = self.accept_operations[service]
         except KeyError:
@@ -126,10 +210,12 @@ class GISView(View):
         param_values = wfs_method.parse_request(self.KVP)
         return wfs_method(**param_values)  # goes into __call__()
 
-    def _get_required_arg(self, argname):
+    def _get_required_arg(self, argname, default=None):
         try:
             return self.KVP[argname]
         except KeyError:
+            if default is not None:
+                return default
             raise MissingParameterValue(argname.lower()) from None
 
     def get_service_description(self, service: str) -> ServiceDescription:
@@ -179,11 +265,15 @@ class WFSView(GISView):
         }
     }
 
+    #: Since URLs to this view are already specifically for WFS,
+    #: allow to omit the service name.
+    default_service = "WFS"
+
     #: Metadata of the capabilities:
     wfs_service_constraints = {
-        "ImplementsBasicWFS": False,  # only simple WFS
-        "ImplementsTransactionalWFS": False,  # only simple WFS
-        "ImplementsLockingWFS": False,  # only simple WFS
+        "ImplementsBasicWFS": True,  # Advanced queries/xpath
+        "ImplementsTransactionalWFS": False,  # only reads
+        "ImplementsLockingWFS": False,  # only basic WFS
         "KVPEncoding": True,  # HTTP GET support
         "XMLEncoding": False,  # HTTP POST requests
         "SOAPEncoding": False,  # no SOAP requests
@@ -222,3 +312,18 @@ class WFSView(GISView):
     def get_feature_types(self) -> List[FeatureType]:
         """Return all available feature types this server exposes"""
         return self.feature_types
+
+    def get_index_context_data(self, **kwargs):
+        """Add WFS specific metadata"""
+        wfs_output_formats = self.accept_operations["WFS"]["GetFeature"].output_formats
+
+        context = super().get_index_context_data(**kwargs)
+        context.update(
+            {
+                "wfs_features": self.get_feature_types(),
+                "wfs_output_formats": wfs_output_formats,
+                "wfs_filter_capabilities": self.wfs_filter_capabilities,
+                "wfs_service_constraints": self.wfs_service_constraints,
+            }
+        )
+        return context
